%calculates Forward Matrix and residual errors for given ssf, cc24 and illuminant
%if sweepFlag is set it keeps the matrix fixed for the reference illuminant
%and varies the input illuminant. if wbflag is set it white balances the
%input to the matrix to [1,1,1], otherwise it lets camera neutral float.
%Uses wavelength range compatible with CC24 reflectances.
wl = 380:730;                                       %wavelength range
sweepFlag = 1;                                      %illuminant sweep if set
wbFlag = 0;                                         %wb each time if set

load matlab                                         %not to load files below
ill = ill_P30;                                      %D53 or P30 for 3000K BB
illType = 'P';                                      %set to 'D' or 'P'
illcct = 3000;                                      %set CCT reference
% ill = illuminant([illType num2str(illcct)],wl)/100; %needs optprop
% cc24 = xlsread('E:\My Drive\Camera\Data\Reflectance\CC24_Classic_BabelColor30_Pre2015.csv','H2:AQ25')';
% cc24 = interp1(380:10:730,cc24,wl);                         %reflectance
% cmf = csvread('E:\My Drive\Camera\Data\CMF\ciexyz31_1.csv');%1931 Obs
% cmf = cmf(21:371,2:4);                                      %380:730nm
% ssf = csvread('E:\My Drive\Camera\Data\CFA\D5100.csv');     %Nikon D5100
% ssf = interp1(380:5:780,ssf(:,2:4),wl);                     %Darrodi et al
%%  Reference Transform: illuminant, cc24 scene, Nikon D5100 camera
xyzn = ill * cmf;                                   %illuminant neutral
xyzn = xyzn/xyzn(2);                                %illuminant White Point
cctDuv(xyzn);                                       %optional check

Ln = cc24 .* ill';                                  %radiance from CC24 target

xyzRef = Ln' * cmf;                                 %reference values for given ill
xyzRef = xyzRef/xyzRef(22,2)*mean(cc24(:,22))/100;  %normalize to reference
labRef = xyz2labn(xyzRef,xyzn);

grey = sum(ssf .* ill');                            %camera neutral: gray card
wbMult0 = 1 ./ grey;                                %Wb multipliers ref
wbMult0 = wbMult0/wbMult0(2);                       %normalized

raw = Ln' * ssf;                                    %raw data from radiance
wbraw = raw .* wbMult0;                             %white balanced raw data
wbraw = wbraw/wbraw(22,2)*xyzRef(22,2);             %normalize to reference

[M,M0,de76,de00,k,output] = estimateMatrixWpp(wbraw,xyzRef,xyzn,'fminunc');
%xyz = wbraw * M * k format.  Remember: Forward Matrix is M' in this case
p = [1:18 22];                                      %only one neutral patch
disp(['M0 de00: ' num2str( mean(deltaE00( xyz2labn(wbraw(p,:)*M0,xyzn), labRef(p,:) )))])

%% - see what happens if illuminant changes but all else stays the same
if sweepFlag                                         %#ok<*UNRCH>
    cctRange = illcct-800:100:illcct+1200;
    mn = zeros(length(cctRange),1);                 %mean, max and other stats
    mx = mn; st = mn; mxP = mn; cct = mn; duv = mn; cctErr = mn;
    fprintf('%s Illuminant, Err:   de00 mean     max    stdv   | de76 mean     max    stdv\n', illType)
    i = 1;
    for T = cctRange
        ill = illuminant([illType num2str(T)], wl); %vary illuminant
        Ln = cc24 .* ill';                          %radiance from CC24
        
        %raw section: camera readings in the field, always available
        raw = Ln' * ssf;                            %raw data from radiance
        grey = sum(ssf .* ill');                    %camera neutral: gray card
        grey = grey / grey(2);                      %changes with illuminant
                                                    
        if wbFlag
            wbraw = raw ./ grey;                    %white balance each time
            WP = sum(M);                            %illuminant neutral WP in xyz
            wbTit = '';
        else
            wbraw = raw .* wbMult0;                 %use original wb multipliers
            WP = grey.*wbMult0*M;                   %illuminant neutral WP in xyz
            wbTit = 'non-';
        end
        WP = WP / WP(2);                            %Estimated White Point
        wbraw = wbraw/wbraw(22,2)*xyzRef(22,2);     %normalize
        
        xyz = wbraw * M;
        
        %reference section: meter readings in the field, mostly not available
        xyzn = ill * cmf;                           
        xyzn = xyzn/xyzn(2);                        %Actual WP
        
        xyzRef = Ln' * cmf;                         %reference values for given ill
        xyzRef = xyzRef/xyzRef(22,2)*mean(cc24(:,22))/100;
        labRef = xyz2labn(xyzRef,xyzn);
        
        de00 = deltaE00( xyz2labn(xyz,xyzn), labRef );
        de76 = deltaE76( xyz2labn(xyz,xyzn), labRef );
        
        %     figure; bar3c(reshape(de00,6,[])',roo2disp(colorchecker))%requires optprop
        %     zlabel('\DeltaE_{00}'); zlim([0 4])
        
        mn(i) = mean(de00(p));
        st(i) = std(de00(p));
        
        [mx(i),mxP(i)] = max(de00);
        
        [cct(i),duv(i)] = cctDuv(WP,0);
        cctErr(i) = deltaE00(xyz2labn(WP,xyzn),xyz2labn(xyzn,xyzn));
        
        fprintf('%u / %.0f, %.2f:     %7.4f %7.4f %7.4f   |   %7.4f %7.4f %7.4f\n',...
            T,cct(i),cctErr(i),mn(i),mx(i),st(i),mean(de76),max(de76),std(de76))
        i = i + 1;
    end
    
    figure; plot(cctRange,mn,cctRange,mx, cctRange,st); grid; grid minor
    ylim([0 4]); hold on;  ylabel('deltaE00')
    xlabel('Correlated Color Temperature of Light at Scene (K)');
    text(cctRange,mx,char(num2str(mxP)),'FontSize',8,'VerticalAlignment','bottom')
    plot(cctRange,cctErr,'.')
    legend('Mean','Max','StdDev','White Point')
    title(['BabelColor30 CC24, 19 Patches, Fixed Transform for ' illType num2str(illcct) ])
end